# $Id$

package raidstatus::mod_3ware;

use warnings;
use strict;

our $name = "mod_3ware";
our @platforms = ("FreeBSD", "Linux");

# use this binary to query information
our @tw_cli_paths = ("/client/sbin/tw_cli", "/usr/sbin/tw_cli", "/usr/local/sbin/tw_cli");
our $tw_cli = "";
our $tw_cli_available = '';

# use CVS/RCS Revision tag to set up the package version
#our $VERSION = do { my @r = (q$Revision$ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
our $VERSION = "git";

sub new {
  my ($class) = @_;
  my $self = {};

  bless ($self,$class);

  # check availability of tw_cli
  foreach my $tw_cli_binary (@tw_cli_paths){
    if ( -e "$tw_cli_binary" ) {
      $tw_cli_available = "yes";
      $tw_cli = $tw_cli_binary;
    }
  }
  return $self;
}

sub getName {
  my ($self) = @_;
  return $name;
}

sub getPlatforms {
  my ($self) = @_;
  return @platforms;
}

sub getUnits {
  my ($self) = @_;
  my @detectedUnits = ();

  if ( $tw_cli_available ){
    # first we need to fetch a list of all 3ware controllers
    my @detectedControllers = ();
    my $cmd = "$tw_cli info";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      # for tw_cli-7
      if ($line =~ m/^Controller\s(\d+)/ ){
        my $controller = $1;
        push(@detectedControllers,"c".$controller);
      }
      # for tw_cli-9 
      if ($line =~ m/^(c\d+)\s+/ ){
        my $controller = $1;
        push(@detectedControllers,$controller);
      } # end if
    } # end while
    close(CMD);

    # fetch all unit Names from each controller
    foreach my $controller (@detectedControllers){
      my $cmd = "$tw_cli info $controller";
      open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
      while (my $line = <CMD>) {
        chomp($line);
        # for tw_cli-7
        if ($line =~ m/^\s*Unit\s(\d+)/ ){
          my $unit = $1;
          push(@detectedUnits, $controller . ":" . "u".$unit);
        } # end if
        # for tw_cli-9
        if ($line =~ m/^(u\d+)\s+/ ){
          my $unit = $1;
          push(@detectedUnits, $controller . ":" . $unit);
        } # end if
      } # end while
      close(CMD);
    } # end foreach
  } # end if tw_cli_available  
  return @detectedUnits;
}

sub checkUnits {
  my ($self, @units) = @_;
  my @unitStatus = ();
  foreach my $unit (@units){
    my ($controller, $unitname) = split(/:/, $unit);
    my $cmd = "$tw_cli info $controller $unitname status";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      if ( $line =~ m!^(\w+)\s! ||
           $line =~ m!^/$controller/$unitname\sstatus\s+=\s+(\w+)! ){
        my $state = $1;
        my $ret = "UNEXPECTED";
        if ($state eq "OK"){
          $ret = "OK";
        } elsif ( $state eq "REBUILDING" || $state eq "INITIALIZING" ) {
          $ret = "WARNING";
        } elsif ( $state eq "DEGRADED" ) { 
          $ret = "CRITICAL";
        }
        push(@unitStatus, "$unit:$ret");
      }
    }
  }
  return @unitStatus;
}

1;

# vim: ai et ts=2 shiftwidth=2

