# $Id$

package raidstatus::mod_arrayinfo;

use warnings;
use strict;

our $name = "mod_arrayinfo";
our @platforms = "Linux";

# use this binary to query information
our $arrayinfo = "/usr/sbin/array-info";

# use CVS/RCS Revision tag to set up the package version
#our $VERSION = do { my @r = (q$Revision$ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
our $VERSION = "git";

sub new {
  my ($class) = @_;
  my $self = {};

  bless ($self,$class);
  return $self;
}

sub getName {
  my ($self) = @_;
  return $name;
}

sub getPlatforms {
  my ($self) = @_;
  return @platforms;
}

sub getUnits() {
  my ($self) = @_;
  my @detectedUnits = ();
  if ( -d "/dev/cciss") {
    if (! -e $arrayinfo ) {
      print("ERROR: HP array detected (/dev/cciss/*), but array-info not installed!\n");
      print(" apt-get install array-info\n");
    }
    else {
      my $cmd = "ls /dev/cciss/";
      open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
      while (my $line = <CMD>) {
        chomp($line);
        if ( $line =~ m/^(c\dd\d)$/ ) {
          my $unit = $1;
          unshift(@detectedUnits, $unit); 
        } # end if
      } # end while
    } # end else
  }  
  close(CMD);

  return @detectedUnits;
}

sub checkUnits() {
  my ($self, @units) = @_;
  my @unitStatus = ();
  foreach my $unit (@units){
    my $cmd = "$arrayinfo -d /dev/cciss/$unit ";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      if ($line =~ m/^\s+Status\s+: Logical drive is\s+(.*)$/ ) {
        my $state = $1;
        my $ret = "UNEXPECTED";
        if ( $state eq "ok" ) {
          $ret = "OK";
	}
	else {
          $ret = "CRITICAL";
        }
        push(@unitStatus, "$unit:$ret");
      } # end if
    } # end while
  } # end foreach  
  return @unitStatus;
}



1;

# vim: ai et ts=2 shiftwidth=2

