# $Id$

package raidstatus::mod_mdraid;

use warnings;
use strict;

our $name = "mod_mdraid";
our @platforms = "Linux";

# use this binary to query information
our $mdadm = "/sbin/mdadm";

# mdstat file
our $mdstat = "/proc/mdstat";

# use CVS/RCS Revision tag to set up the package version
#our $VERSION = do { my @r = (q$Revision$ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
our $VERSION = "git";

sub new {
  my ($class) = @_;
  my $self = {};

  bless ($self,$class);
  return $self;
}

sub getName {
  my ($self) = @_;
  return $name;
}

sub getPlatforms {
  my ($self) = @_;
  return @platforms;
}

sub getUnits() {
  my ($self) = @_;
  my @detectedUnits = ();
  if ( -e "$mdstat" ) {
    my $cmd = "cat $mdstat";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      if ( $line =~ m/^(md\d+)\s+:/ ) {
        my $unit = $1;
        unshift(@detectedUnits, $unit); 
      } # end if
    } # end while
  }  
  close(CMD);

  return @detectedUnits;
}

sub checkUnits() {
  my ($self, @units) = @_;
  my @unitStatus = ();
  foreach my $unit (@units){
    my $cmd = "$mdadm --query --detail /dev/$unit";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      if ($line =~ m/^\s+State\s+:\s+(.*)$/ ) {
        my $state = $1;
        $state =~ s/^\s+|\s+$//g ;
        my $ret = "UNEXPECTED";
        if ( $state eq "active" or $state eq "clean" ) {
          $ret = "OK";
        } elsif ( $state =~ /recovering/ || 
                  $state =~ /resyncing/ ) {
          $ret = "WARNING";
        } elsif ( $state =~ /degraded/ ) {
          $ret = "CRITICAL";
        }
        push(@unitStatus, "$unit:$ret");
      } # end if
    } # end while
  } # end foreach  
  return @unitStatus;
}



1;

# vim: ai et ts=2 shiftwidth=2

