# $Id$

package raidstatus::mod_megacli;

use warnings;
use strict;

our $name = "mod_megacli";
our @platforms = ("Linux");

# use this binary to query information
our @megacli_paths = ("/usr/sbin/megacli");
our $megacli = "";
our $megacli_available = '';

# use CVS/RCS Revision tag to set up the package version
#our $VERSION = do { my @r = (q$Revision$ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
our $VERSION = "git";

sub new {
  my ($class) = @_;
  my $self = {};

  bless ($self,$class);

  # check availability of megacli
  foreach my $megacli_binary (@megacli_paths){
    if ( -e "$megacli_binary" ) {
      $megacli_available = "yes";
      $megacli = $megacli_binary;
    }
  }
  return $self;
}

sub getName {
  my ($self) = @_;
  return $name;
}

sub getPlatforms {
  my ($self) = @_;
  return @platforms;
}

sub getUnits {
  my ($self) = @_;
  my @detectedUnits = ();

  if ( $megacli_available ){
    # first we need to fetch a list of all MegaRAID controllers
    my $nControllers = 0;
    my @detectedControllers = ();
    my $cmd = "$megacli -adpCount -NoLog";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      if ($line =~ m/^Controller Count: (\d+)\./ ){
        $nControllers = $1;
      } # end if
    } # end while
    close(CMD);

    # fetch all unit Names from each controller
    for (my $cid = 0; $cid < $nControllers; $cid++) {
      my $cmd = "$megacli -LDInfo -Lall -a$cid -NoLog";
      open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
      while (my $line = <CMD>) {
        chomp($line);
        if ($line =~ m/^Virtual Disk: (\d+)/ ){
          my $unit = $1;
          push(@detectedUnits, $cid . ":" . $unit);
        } # end if
      } # end while
      close(CMD);
    } # end foreach
  } # end if megacli_available  
  return @detectedUnits;
}

sub checkUnits {
  my ($self, @units) = @_;
  my @unitStatus = ();
  foreach my $unit (@units){
    my ($controller, $unitname) = split(/:/, $unit);
    my $cmd = "$megacli -LDInfo -L$unitname -a$controller -NoLog";
    open(CMD, "$cmd|") or die("couldn't open $cmd|: $!");
    while (my $line = <CMD>) {
      chomp($line);
      if ( $line =~ m!^State: (\w+)! ) {
        my $state = $1;
        my $ret = "UNEXPECTED";
        if ($state eq "Optimal"){
          $ret = "OK";
        } else {
          $ret = "CRITICAL";
        }
        push(@unitStatus, "$unit:$ret");
      }
    }
  }
  return @unitStatus;
}

1;

# vim: ai et ts=2 shiftwidth=2

